-----------------------------------------------------------
-- Diplomatic action evaluation
-- 
-----------------------------------------------------------


require('utils')

function CalculateAlignmentFactor(ai, country1, country2)
	local dist = ai:GetCountryAlignmentDistance( country1, country2 ):Get()
	return math.min(dist / 400.0, 1.0)
end

function DiploScore_InfluenceNation(ai, actor, recipient, observer)
	if observer == actor then
		local recipientCountry = recipient:GetCountry()
		local actorCountry = actor:GetCountry()
		local actorFaction = actorCountry:GetFaction()
		local dist = ai:GetNormalizedAlignmentDistance(recipientCountry, actorFaction):Get()
		
		--Utils.LUA_DEBUGOUT("====================================================")
		
		--Utils.LUA_DEBUGOUT("Align dist: " .. tostring(actor) .. " -> " .. tostring(recipient) .. " = " .. dist )
		-- close to joining our faction?
		if dist > 1.0 and dist < 7.0 then
			local factor = 1.0 - ( (dist - 1.0)/ (7.0 - 1) )
			factor = 100 * factor
			--Utils.LUA_DEBUGOUT("factor: " .. factor)
			if recipientCountry:HasNeighborInFaction(actorFaction) then
				factor = factor + 20
				--Utils.LUA_DEBUGOUT("faction neighbor factor: " .. factor)
			end
			
			-- calculate importance
			local importanceFactor = 0
			local maxIC = recipientCountry:GetMaxIC()
			local ourMaxIC = actorCountry:GetMaxIC()
			if maxIC > ourMaxIC * 5.0 then
				importanceFactor = importanceFactor + 70
			elseif maxIC > ourMaxIC * 2.0 then
				importanceFactor = importanceFactor + 50
			elseif maxIC > ourMaxIC then
				importanceFactor = importanceFactor + 30
			elseif maxIC > ourMaxIC* 0.75 then
				importanceFactor = importanceFactor + 20
			elseif maxIC > ourMaxIC* 0.5 then
				importanceFactor = importanceFactor + 10
			elseif maxIC < ourMaxIC * 0.1 then
				importanceFactor = importanceFactor - 40
			end
			--Utils.LUA_DEBUGOUT("importanceFactor: " .. importanceFactor)
			
			-- factor neutrality
			local effectiveNeutrality = recipientCountry:GetEffectiveNeutrality():Get()
			if effectiveNeutrality > 90 then
				importanceFactor = importanceFactor - 100
			elseif effectiveNeutrality > 80 then
				importanceFactor = importanceFactor - 70
			elseif effectiveNeutrality > 70 then
				importanceFactor = importanceFactor - 10
			elseif effectiveNeutrality < defines.diplomacy.JOIN_FACTION_NEUTRALITY+10 then
				importanceFactor = importanceFactor + 50
			end
			--Utils.LUA_DEBUGOUT("importanceFactor - neutralitycalcs: " .. importanceFactor)
			
			factor = factor + importanceFactor
			--Utils.LUA_DEBUGOUT("factor: " .. factor)
			return Utils.CallScoredCountryAI(actor, 'DiploScore_InfluenceNation', factor, ai, actor, recipient, observer)
		elseif dist > 12.0 then
			--if false then
			-- close to other faction?
			--Utils.LUA_DEBUGOUT("clsoe to other faction test")
			for faction in CCurrentGameState.GetFactions() do
				if faction:IsValid() and not (faction == actorFaction) then
					dist = ai:GetNormalizedAlignmentDistance(recipientCountry, faction):Get()
					--Utils.LUA_DEBUGOUT("Align dist: " .. tostring(recipient) .. " -> " .. tostring(faction:GetFactionLeader()) .. " = " .. dist )
					if dist > 1.0 and dist < 2.0 then
						--Utils.LUA_DEBUGOUT("reverse check")
						local factionLeader = faction:GetFactionLeader()
						local score = DiploScore_InfluenceNation(ai, factionLeader, recipient, factionLeader)
						score = score * 0.75 -- penalize a bit to boost personal faction members
						if score > 50 then					
							if ( recipientCountry:CalculateNumberOfActiveInfluences() > 1 ) then
								score = 0 -- dont overdo it
							end
						end
						return score
					end	
				end
			end
			--end
			--Utils.LUA_DEBUGOUT("not close to anyone...")
			
			-- neighbor?
			--if recipientCountry:IsNeighbour(actor) then
			--	local score = 0
			--	local rel = ai:GetRelation(recipient, actor)
			--	local strategy = recipient:GetCountry():GetStrategy()
				--score = score + strategy:GetFriendliness(actor) / 4
				--score = score - strategy:GetAntagonism(actor) / 4
			--score = score - rel:GetThreat():Get() / 2
			--
			--	if score > 0 then
			--		local relation = rel:GetValue():GetTruncated()
			--		score = score + relation / 2
			--	end
				
			--	if score > 65 then
			--		score = 65 -- dont want to overshadow other juicy targets
			--	end
			--
			--	return Utils.CallScoredCountryAI(actor, 'DiploScore_InfluenceNation', score, ai, actor, recipient, observer)
			--end
		end
		--Utils.LUA_DEBUGOUT("====================================================")
		return Utils.CallScoredCountryAI(actor, 'DiploScore_InfluenceNation', 0, ai, actor, recipient, observer)
	else
		return 100 -- we cant respond to this
	end
end

-- for debugging
--local GOODS_TO_STRING = { [0] = "_SUPPLIES_","_FUEL_",	"_MONEY_",	"_CRUDE_OIL_",	"_METAL_",	"_ENERGY_",	"_RARE_MATERIALS_" }

function DiploScore_OfferTrade(ai, actor, recipient, observer, action)
	if observer == actor then
		return 100 -- handled from foreign minister
	else -- evaluate offer of trade from actor
		--Utils.LUA_DEBUGOUT("TRADE ========")
		--Utils.LUA_DEBUGOUT("actor: " .. tostring( actor ))
		--Utils.LUA_DEBUGOUT("recipient: " .. tostring( recipient ))
		--if ai:CanTradeFreeResources(actor, recipient) then
		--	Utils.LUA_DEBUGOUT("CanTradeFreeResources")
		--end
		local score = 0
		local actorCountry = actor:GetCountry()
		local recipientCountry = recipient:GetCountry()
		local route = action:GetRoute()
		
		if ai:AlreadyTradingResourceOtherWay( action:GetRoute() ) 
		or ai:AlreadyTradingDisabledResource( action:GetRoute() )
		then
			--Utils.LUA_DEBUGOUT("+x+x++x+x+xalready trading that+x+x++x+x+x")
			return 0
		end
		
		--Utils.LUA_DEBUGOUT("from: " .. tostring( route:GetFrom() ))
		--Utils.LUA_DEBUGOUT("to: " .. tostring( route:GetTo() ))
		
		local MAX_GOODS = CGoodsPool._GC_NUMOF_-1
		for goods = 0, MAX_GOODS do
			local balance = recipientCountry:GetDailyBalance(goods):Get()
			if goods == CGoodsPool._SUPPLIES_ then
				balance = math.min( balance, recipientCountry:GetSupplyBalanceAverage():Get() )
			elseif goods == CGoodsPool._MONEY_ then
				if ai:CanTradeFreeResources(actor, recipient) then
					balance = 1000					
				else
					balance = math.min( balance, recipientCountry:GetMoneyBalanceAverage():Get() )
				end
			end
			
			local absBalance = math.abs(balance)
			local inTrade = route:GetTradedFromOf(goods):Get()
			local outTrade = route:GetTradedToOf(goods):Get()
			
			--if ai:CanTradeFreeResources(actor, recipient) then
			--	Utils.LUA_DEBUGOUT("_______PRETRADE: " .. GOODS_TO_STRING[goods] .. "_________")
			--	Utils.LUA_DEBUGOUT("absBalance - " .. absBalance)
			--	Utils.LUA_DEBUGOUT("inTrade - " .. inTrade)
			--	Utils.LUA_DEBUGOUT("outTrade - " .. outTrade)
			--	Utils.LUA_DEBUGOUT("_______*************_______")
			--end
					
			
			if absBalance > 0.005 and (outTrade > 0.01 or inTrade > 0.01) then
				--Utils.LUA_DEBUGOUT("____________goods : " .. GOODS_TO_STRING[goods] )
				--Utils.LUA_DEBUGOUT("balance : " .. balance )
				
				if inTrade > 0.001 and absBalance > 0.001 then
					--Utils.LUA_DEBUGOUT("GetTradedToOf: " .. inTrade)
					local tradeFactor = 0.0
					if balance < 0.0 then
						tradeFactor = math.min(inTrade / (-balance) * 2, 1.0)
						score = score + math.min(tradeFactor, 1.0) * 100
						--Utils.LUA_DEBUGOUT("tradefactor: " .. tradeFactor)
						--Utils.LUA_DEBUGOUT("inTrade: " .. inTrade)
						--Utils.LUA_DEBUGOUT("balance: (-)" .. balance)
					else
						if not (goods == CGoodsPool._MONEY_) then
							tradeFactor = math.min(inTrade / balance, 1.0) * 0.5
							score = score - math.min(tradeFactor * 2, 1.0) * 100
							--Utils.LUA_DEBUGOUT("tradefactor: " .. tradeFactor)
							--Utils.LUA_DEBUGOUT("inTrade: " .. inTrade)
							--Utils.LUA_DEBUGOUT("balance: ()" .. balance)
						end
					end
					
					--Utils.LUA_DEBUGOUT("tradefactor: " .. tradeFactor)
					--Utils.LUA_DEBUGOUT("score: " .. score)
				end

				if outTrade > 0.001 then
					--Utils.LUA_DEBUGOUT("GetTradedFromOf : " .. outTrade )
					--local tradeFactor = 0
					--if (balance > 0) then
					--	tradeFactor = (absBalance - outTrade) / absBalance
					--else
					--	score = 0
					--end
					--score = score + tradeFactor * 100
					
					if balance > 0.0 then
						tradeFactor = math.min(outTrade / (balance), 1.0)
						if tradeFactor > 0.95 then
							if tradeFactor > 0.999999 then
								tradeFactor = 0
							else
								tradeFactor = (0.06 - tradeFactor - 0.95)
							end	
							score = score + math.min(tradeFactor, 1.0) * 100
						elseif tradeFactor > 0.7 then
							score = score + 85
						elseif tradeFactor > 0.3 then
							score = score + 75
						elseif tradeFactor > 0.001 then
							score = score + 60
						end
						--score = score + math.min(tradeFactor, 1.0) * 100
						--Utils.LUA_DEBUGOUT("tradefactorz: " .. tradeFactor)
						--Utils.LUA_DEBUGOUT("outTrade: " .. outTrade)
						--Utils.LUA_DEBUGOUT("balance: (-)" .. balance)
					else
						--tradeFactor = math.min(outTrade / balance, 1.0) * 0.5
						score = score - 1000 -- math.min(tradeFactor * 2, 1.0) * 100
						--Utils.LUA_DEBUGOUT("tradefactor: " .. tradeFactor)
						--Utils.LUA_DEBUGOUT("outTrade: " .. outTrade)
						--Utils.LUA_DEBUGOUT("balance: (-)" .. balance)
					end
					
					
					--Utils.LUA_DEBUGOUT("tradefactor: " .. tradeFactor)
					--Utils.LUA_DEBUGOUT("score: " .. score)
				elseif inTrade > 0.01 and ai:CanTradeFreeResources(actor, recipient) then
					-- free stuff!
					score = 100
				end

				--Utils.LUA_DEBUGOUT("____________")
			end
		end
		
		-- we need transports to trade
		if actorCountry:NeedConvoyToTradeWith( recipient ) then
			if route:GetConvoyResponsible() == recipient then
				if recipientCountry:GetTransports() == 0 then
					score = 0
				end
			end
		end
		
		--Utils.LUA_DEBUGOUT("score before strategy" .. score)
		
		if score > 0.001 or score < -0.001 then
					
			if score > 30 then			
				local rel = ai:GetRelation(recipient, actor)
				local strategy = recipient:GetCountry():GetStrategy()
				score = score - strategy:GetAntagonism(actor) / 15				
				score = score + strategy:GetFriendliness(actor) / 15
				score = score - rel:GetThreat():Get() / 2
				if rel:IsGuaranteed() then
					score = score + 5
				end
				if rel:HasFriendlyAgreement() then
					score = score + 10
				end
				if rel:AllowDebts() then
					score = score + 5
				end
				if rel:IsFightingWarTogether() then
					score = score + 15
				end
				
				--Utils.LUA_DEBUGOUT("GetAntagonism" .. tostring(strategy:GetAntagonism(actor)))
				--Utils.LUA_DEBUGOUT("GetFriendliness" .. tostring(strategy:GetFriendliness(actor)))
				--Utils.LUA_DEBUGOUT("GetThreat" .. tostring(rel:GetThreat():Get()))
				--Utils.LUA_DEBUGOUT("score after country stuff" .. score)
				local relation = rel:GetValue():GetTruncated()
				score = score + relation / 5
			

			end
			--Utils.LUA_DEBUGOUT("score after country stuff" .. score)
		end
		--Utils.LUA_DEBUGOUT("===================\n")
		return Utils.CallScoredCountryAI(recipient, 'DiploScore_OfferTrade', score, ai, actor, recipient, observer)
		
	end
end


function DiploScore_InviteToFaction(ai, actor, recipient, observer)
	if observer == actor then -- are recipient worth inviting
		local recipientCountry = recipient:GetCountry()
		if recipientCountry:IsAtWar() then
			-- is our war target at war with the faction
			for diploStatus in recipientCountry:GetDiplomacy() do
				local target = diploStatus:GetTarget()
				if target:IsValid() and diploStatus:HasWar() then
					if actor:GetCountry():GetRelation(target):HasWar() then
						return 100 -- at war with our enemies
					elseif target:GetCountry():IsFactionLeader() then
						return 0 -- dont pull us into war against another faction we arent already
					end
				end					
			end
			return 100 -- at war with someone is no problem
		else
			return 100 -- if they can, let the,
		end
	else -- do we, recipient want to accept invite to faction
		local score = 100
		--Utils.LUA_DEBUGOUT("-------------------------------------")
		--Utils.LUA_DEBUGOUT("DiploScore_InviteToFaction (" .. tostring( actor )  .. "->" .. tostring( recipient ) .. ")")
		local faction = actor:GetCountry():GetFaction()
		
		if recipient:GetCountry():IsNeighbourToFactionHostile(faction, true) 
		and (not recipient:GetCountry():HasNeighborInFaction(faction))
		then
			score = 0 -- need some backup bro
		end
		
		return Utils.CallScoredCountryAI(recipient, 'DiploScore_InviteToFaction', score, ai, actor, recipient, observer)
	end
end


function DiploScore_Guarantee(ai, actor, recipient, observer)
	local score = 0

	if observer == actor then
		local actorCountry = actor:GetCountry()
		local recipientCountry = recipient:GetCountry()
		if actorCountry:HasFaction() and actorCountry:GetFaction() == recipientCountry:GetFaction() then
			return 0 -- pointless
		end
		
		if actorCountry:IsGovernmentInExile() then
			return 0 -- pointless
		end
		
		local strategy = actor:GetCountry():GetStrategy()
		score = score + strategy:GetFriendliness(recipient) / 2
		score = score + strategy:GetProtectionism(recipient)
		score = score - strategy:GetAntagonism(recipient) / 2
		score = score - actor:GetCountry():GetDiplomaticDistance(recipient):GetTruncated() 
		
	end

	score = Utils.CallScoredCountryAI(actor, 'DiploScore_Guarantee', score, ai, actor, recipient, observer)
	return score
end


function DiploScore_Embargo(ai, actor, recipient, observer)
	if observer == actor then
		local score = 0
		local actorCountry = actor:GetCountry() 
		local recipientCountry = recipient:GetCountry() 

		if actorCountry:IsAtWar() then
			for enemy in actorCountry:GetCurrentAtWarWith() do
				if recipient:GetCountry():IsFriend(enemy, true) then
					--Utils.LUA_DEBUGOUT( "embargo score " .. tostring(actor) .. " -> " .. tostring(recipient) .. " = " .. 100 )
					return 80 -- fighting our friends
				end
			end
		end
		--Utils.LUA_DEBUGOUT( "embargo score " .. tostring(actor) .. " -> " .. tostring(recipient) .. " = " .. score )
		-- dont use up the last of our points for this
		if actorCountry:GetDiplomaticInfluence():Get() < (defines.diplomacy.EMBARGO_INFLUENCE_COST + 2) then
			score = score / 2 - 1
		end
		--Utils.LUA_DEBUGOUT( "embargo score " .. tostring(actor) .. " -> " .. tostring(recipient) .. " = " .. score )
		return Utils.CallScoredCountryAI(actor, 'DiploScore_Embargo', score, ai, actor, recipient, observer)
	else
		return 0 -- cant respond to this action
	end
end


function DiploScore_NonAgression(ai, actor, recipient, observer)
	if observer == actor then -- we demand nap with recipient
		return DiploScore_NonAgression(ai, recipient, actor, observer)
	else -- actor demands nap with us
		local score = 0
		local rel = ai:GetRelation(recipient, actor)
		local relation = 100 + rel:GetValue():GetTruncated()
		
		if relation > 150 then -- we like them
			score = score + (relation - 150)
		elseif ai:GetNumberOfOwnedProvinces(actor) / 2 >
		       ai:GetNumberOfOwnedProvinces(recipient) then -- much bigger than us
			score = score + 5 + relation / 5
		end
		
		local recipientCountry = recipient:GetCountry()
		local strategy = recipientCountry:GetStrategy()
		score = score + strategy:GetFriendliness(actor) / 4
		score = score - strategy:GetAntagonism(actor) / 4
		--score = score + strategy:GetThreat(actor) / 4
		
		if not recipientCountry:IsNeighbour( actor ) then
			score = score / 2
		end

		score = score - recipientCountry:GetDiplomaticDistance(actor):GetTruncated() 
		--if score > 0 then
			--Utils.LUA_DEBUGOUT("NAP score: " .. score .. " for " .. tostring(actor) .. " - " .. tostring(recipient) )
			--Utils.LUA_DEBUGOUT("friendlyness: " .. strategy:GetFriendliness(actor) ) 
			--Utils.LUA_DEBUGOUT("antagonism: " .. strategy:GetAntagonism(actor) ) 
			--Utils.LUA_DEBUGOUT("threat: " .. strategy:GetThreat(actor) ) 
			--Utils.LUA_DEBUGOUT("d. dist: " ..  recipientCountry:GetDiplomaticDistance(actor):GetTruncated() ) 
			--Utils.LUA_DEBUGOUT("------------------------")
		--end

		return Utils.CallScoredCountryAI(recipient, 'DiploScore_NonAgression', score, ai, actor, recipient, observer)
	end
end

function DiploScore_DemandMilitaryAccess(ai, actor, recipient, observer)
	if observer == actor then -- we demand access of recipient
		local actorCountry = actor:GetCountry()
		local strategy = actorCountry:GetStrategy()
		local score = strategy:GetAccessScore(recipient)

		return score
	else -- actor demands access from us
		--Utils.LUA_DEBUGOUT("DiploScore_DemandMilitaryAccess_________________")
		local score = 0
		local rel = ai:GetRelation(recipient, actor)
		if rel:HasWar() then
			return 0
		end
        
		-- much bigger than us and bordering
        local actorCountry = actor:GetCountry()
		if ( ai:GetNumberOfOwnedProvinces(actor) / 5 > ai:GetNumberOfOwnedProvinces(recipient) )
        and actorCountry:IsNeighbour( recipient )
        then 
			-- if we are not in faction and they are at war
            if actorCountry:IsAtWar() 
            and not (recipient:GetCountry():HasFaction())
            then
                score = 50
            end
		end

		if rel:HasAlliance() then
			score = 80
		end
		
		return Utils.CallScoredCountryAI(recipient, 'DiploScore_DemandMilitaryAccess', score,ai, actor, recipient, observer)
	end
end

function DiploScore_OfferMilitaryAccess(ai, actor, recipient, observer, action)
	local score = 0
	if observer == actor then --should we offer access to recipient
		local rel = ai:GetRelation(actor, recipient)
		if rel:HasWar() then
			return 0
		end
		if actor:GetCountry():HasFaction() then
			return 0
		end
		if actor:GetCountry():GetEffectiveNeutrality():Get() > 70 then
			return	0
		end

		local recipientCountry = recipient:GetCountry()
		if recipientCountry:IsNeighbour( actor ) then -- only for neighbors
			local strategy = actor:GetCountry():GetStrategy()
			score = ( rel:GetValue():GetTruncated()) / 4
			score = score + strategy:GetFriendliness(recipient) / 4
			score = score - strategy:GetAntagonism(recipient) / 4
			score = score - rel:GetThreat():Get()

			if not recipientCountry:IsAtWar() then
				score = score / 4 -- why would we if they dont need it
			end
		end
	else
		score = 100
 	end

	score = Utils.CallScoredCountryAI(recipient, 'DiploScore_OfferMilitaryAccess', score, ai, actor, recipient, observer, action)
	return score
end

function DiploScore_Alliance(ai, actor, recipient, observer, action)
	if observer == actor then 
       		local recipientCountry = recipient:GetCountry()
		local actorCountry = actor:GetCountry()
		local strategy = actorCountry:GetStrategy()
		
		if recipientCountry:IsFactionLeader() then -- as a faction leader we dont want alliances, we want faction members
			return 0
		end

		
		return strategy:GetFriendliness(recipient)
	else 
		local rel = ai:GetRelation(recipient, actor)
		local relation = 200 + rel:GetValue():GetTruncated()
		
		if relation < 100 then
			return 0
		end
		
		local recipientCountry = recipient:GetCountry()
		local actorCountry = actor:GetCountry()
		
		local score = relation / 12.0
		
				
		if actorCountry:IsFactionLeader() then -- as a faction leader we dont want alliances, we want faction members
			return 0
		end
		
		-- check location
		if not recipientCountry:IsNeighbour(actor) then
			-- check if on same continent first
			local recipientContinent = recipientCountry:GetActingCapitalLocation():GetContinent()
			local actorContinent = actorCountry:GetActingCapitalLocation():GetContinent()
			if not (recipientContinent == actorContinent) then
				score = score / 2
			end
		end
		
		-- check their wars
		if actorCountry:IsAtWar() then
			local bMutualEnemies = false
			for enemy in actorCountry:GetCurrentAtWarWith() do
				if recipientCountry:IsEnemy(enemy) then -- use threat as well?
					bMutualEnemies = true
				elseif recipientCountry:IsFriend(enemy, false) then
					return 0 -- fighting our friends
				end
			end
			
			if bMutualEnemies then
				score = score + 20
			else
				score = score / 2 -- better wait until they sorted their problems
			end
		end

		score = score - recipientCountry:GetDiplomaticDistance(actor):GetTruncated() / 10

		local strategy = recipientCountry:GetStrategy()
		score = score + strategy:GetFriendliness(actor) / 2
		score = score - strategy:GetAntagonism(actor) / 2
		score = score - rel:GetThreat():Get() / 2
	

		return Utils.CallScoredCountryAI(recipient, 'DiploScore_Alliance', score, ai, actor, recipient, observer, action)
	end
end


function CalculateWarDesirability(ai, country, target)
	local score = 0
	local countryTag = country:GetCountryTag()
	local targetCountry = target:GetCountry()
	local strategy = country:GetStrategy()

	-- can we even declare war?
	if not ai:CanDeclareWar( countryTag, target ) then
	  return 0
	end

	--Utils.LUA_DEBUGOUT("we can declare war: " .. tostring(minister:GetCountryTag()) .. " -> " .. tostring(target) )


	local antagonism = strategy:GetAntagonism(target);
	local friendliness = strategy:GetFriendliness(target);

	-- dont declare war on people we like
	if friendliness > 0 and antagonism < 1 then
		return 0
	end

	-- no suicide :S
	if country:GetNumberOfControlledProvinces() < targetCountry:GetNumberOfControlledProvinces() / 4 then
		return 0
	end

	-- watch out if we have bad intel, should be infiltrating more
	local intel = CAIIntel(countryTag, target)
	if intel:GetFactor() < 0.1 then
		return 0
	end

	-- compare military power
	local theirStrength = intel:CalculateTheirPercievedMilitaryStrengh()
	local ourStrength = intel:CalculateOurMilitaryStrength()
	local strengthFactor = ourStrength / theirStrength

	if strengthFactor < 1.0 then
		score = score - 75 * (1.0 - strengthFactor)
	else
		score = score + 20 * (strengthFactor - 1.0)
	end

	-- personality
	if strategy:IsMilitarist() then
		score = score * 1.3
	end
	
	return Utils.CallScoredCountryAI(countryTag, 'CalculateWarDesirability', score, ai, country, target)

end

function DiploScore_PeaceAction(ai, actor, recipient, observer, action)
	if observer == actor then
		return 0
	else
		score = 0
		
		-- intel first
		--Utils.LUA_DEBUGOUT("----------")
		local intel = CAIIntel(recipient, actor)
		if intel:GetFactor() > 0.1 then
			local recipientStrength = intel:CalculateTheirPercievedMilitaryStrengh()
			local actorStrength = intel:CalculateOurMilitaryStrength()
			local strengthFactor = actorStrength / recipientStrength - 0.5
			score = 100 * strengthFactor
		end
		--Utils.LUA_DEBUGOUT("score: " .. score )
		
		local sizeFactor = actor:GetCountry():GetNumberOfControlledProvinces() / recipient:GetCountry():GetNumberOfControlledProvinces()
		--Utils.LUA_DEBUGOUT("sizeFactor: " .. sizeFactor )
		sizeFactor = (sizeFactor - 1) * 100
				
		score = score + math.min(sizeFactor, 100)
		
		score = score + recipient:GetCountry():GetSurrenderLevel():Get() * 100
		--Utils.LUA_DEBUGOUT("score: " .. score )
		score = score - actor:GetCountry():GetSurrenderLevel():Get() * 100
		--Utils.LUA_DEBUGOUT("score: " .. score )
		
		local strategy = recipient:GetCountry():GetStrategy()
		score = score + strategy:GetFriendliness(actor) / 2
		score = score - strategy:GetAntagonism(actor) / 2
		--score = score + strategy:GetThreat(actor) / 2
		--Utils.LUA_DEBUGOUT("score: " .. score )
		return score
	end
end

function DiploScore_SendExpeditionaryForce(ai, actor, recipient, observer, action)
	if observer == actor then
		return 0 
	else
		-- do we want to accept?
		local recipientCountry = recipient:GetCountry()
		if recipientCountry:GetDailyBalance( CGoodsPool._SUPPLIES_ ):Get() > 1.0 then
			local  score = 0
			-- maybe we have enough stockpiles
			local supplyStockpile = recipientCountry:GetPool():Get( CGoodsPool._SUPPLIES_ ):Get()
			local weeksSupplyUse = recipientCountry:GetDailyExpense( CGoodsPool._SUPPLIES_ ):Get() * 7
			if supplyStockpile > weeksSupplyUse * 20.0 then
				score = score + 70
			elseif supplyStockpile > weeksSupplyUse * 10.0 then
				score = score + 40
			end
			
			if recipientCountry:IsAtWar() then
				score = score + 20
			else
				score = 0 -- no war, no need for troops
			end
			
			return score
		else
			local score = 0
			-- maybe we have enough stockpiles
			local supplyStockpile = recipientCountry:GetPool():Get( CGoodsPool._SUPPLIES_ ):Get()
			local weeksSupplyUse = recipientCountry:GetDailyExpense( CGoodsPool._SUPPLIES_ ):Get() * 7
			if supplyStockpile > weeksSupplyUse * 24.0 then
				score = score + 70
			elseif supplyStockpile > weeksSupplyUse * 12.0 then
				score = score + 40
			end

			if recipientCountry:IsAtWar() then
				score = score + 20
			else
				score = 0 -- no war, no need for troops
			end
			
		end
		return 0
	end
end

function DiploScore_LicenceTechnology(ai, actor, recipient, observer, action)
	if observer == actor then
		return 0 
	else
		--Utils.LUA_DEBUGOUT("LICENS ------------------------------")
		if not action:GetSubunit() then
			return 0
		end
	
		local score = 0
		local actorCountry = actor:GetCountry()
		local recipientCountry = recipient:GetCountry()
		local rel = ai:GetRelation(recipient, actor)
		
		if rel:GetValue():GetTruncated() < 0 then
			return 0
		end
		
		if rel:HasWar() then
			return 0
		end
		--Utils.LUA_DEBUGOUT("1 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		-- we can give tech to
		-- - people in faction
		-- - people in alliance
		-- - people fighting our enemies
		-- - people close in triangle (not far away! scale price here too)
		local allied = false
		if ( recipientCountry:HasFaction() and recipientCountry:GetFaction() == actorCountry:GetFaction() ) then
			score = score + 70
			allied = true
		elseif rel:HasAlliance() then
			score = score + 60
			allied = true
		end
		--Utils.LUA_DEBUGOUT("2 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		local fightingFriend = false
		local fightingEnemy = false
		if actorCountry:IsAtWar() then
			for enemy in actorCountry:GetCurrentAtWarWith() do
				if recipientCountry:IsEnemy(enemy) then
					--Utils.LUA_DEBUGOUT("mutual enemy: " .. tostring(enemy) .. " for " ..  tostring(actor) .. " and " . .tostring(recipient))
					fightingEnemy = true
					--Utils.LUA_DEBUGOUT("mutual enemy")
				elseif recipientCountry:IsFriend(enemy, true)
				and ai:GetRelation(enemy, actor):GetValue():GetTruncated() > 20
				then
					fightingFriend = true
				end
			end
		end
		
		if fightingFriend then
			return 0
		end
		--Utils.LUA_DEBUGOUT("3 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		if fightingEnemy then
			if not allied then
				score = 20 -- need some base
			end
			score = score + 30
			--Utils.LUA_DEBUGOUT("fightingenemy")
		else
			--Utils.LUA_DEBUGOUT("factor: " .. CalculateAlignmentFactor(ai, actorCountry, recipientCountry) * 50)
			score = score - CalculateAlignmentFactor(ai, actorCountry, recipientCountry) * 50
			
		end
		--Utils.LUA_DEBUGOUT("4 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		local threat = rel:GetThreat():Get()
		score = score - threat * CalculateAlignmentFactor(ai, actorCountry, recipientCountry)
		--Utils.LUA_DEBUGOUT("5 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		-- consider the money
		local offeredMoney = action:GetMoney():Get()
		local moneyPile = math.max( recipientCountry:GetPool():Get( CGoodsPool._MONEY_ ):Get(), 100.0)
		if offeredMoney > moneyPile * 0.3 then
			score = score + 30
		elseif offeredMoney > moneyPile * 0.2 then
			score = score + 20
		elseif offeredMoney > moneyPile * 0.1 then
			score = score + 10
		elseif offeredMoney < moneyPile * 0.03 then
			score = score - 30
		else
			score = score - 15
		end
		
		--Utils.LUA_DEBUGOUT("6 LICENS " .. tostring(actor) .. " -> " ..  tostring(recipient) .. " = " .. score)
		--Utils.LUA_DEBUGOUT("LICENS ------------------------------")
		return score
	end
end

function DiploScore_Debt(ai, actor, recipient, observer)
	local actorCountry = actor:GetCountry()
	local recipientCountry = recipient:GetCountry()
	
	if observer == actor then
		if recipientCountry:IsAtWar() 
		and ( recipientCountry:HasFaction() and recipientCountry:GetFaction() == actorCountry:GetFaction() )
		then
			if actorCountry:GetPool():Get( CGoodsPool._MONEY_ ):Get() < 10 
			and actorCountry:GetDailyBalance(CGoodsPool._MONEY_):Get() < 0
			then 
				return 100
			else
				return 0
			end
		else
			return 0
		end
	else
		if recipientCountry:IsAtWar() 
		and ( recipientCountry:HasFaction() and recipientCountry:GetFaction() == actorCountry:GetFaction() )
		then
			local recipientMoney = recipientCountry:GetPool():Get( CGoodsPool._MONEY_ ):Get()
			local actorMoney = actorCountry:GetPool():Get( CGoodsPool._MONEY_ ):Get()
			if (recipientMoney * 5 > actorMoney) 
			and (recipientMoney > 200) then 
				return 100
			else
				return 0
			end
		else
			return 0
		end
	end
end


function DiploScore_CallAlly(ai, actor, recipient, observer, action)
	if observer == actor then
		return 100
	else
		local actorCountry = actor:GetCountry()
		local recipientCountry = recipient:GetCountry()
		
		if actorCountry:GetFaction() == recipientCountry:GetFaction() then
			return 100
		elseif recipientCountry:GetOverlord() == actor then
			return 100
		else
			local score = DiploScore_Alliance(ai, actor, recipient, observer, nil)
			if score < 50 then
				return 40
			else
				return 100
			end
		end
	end
end
